unit RoundPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TCustomRoundPanel = class(TCustomControl)
  private
    FPen: TPen;
    FBrush: TBrush;
    FArc: Integer;
    FColor: TColor;
    FAutoSizeDocking: Boolean;
    FFullRepaint: Boolean;
    FLocked: Boolean;
    FAlignment: TAlignment;
    procedure SetBrush(Value: TBrush);
    procedure SetPen(Value: TPen);
    procedure SetArc(Value: Integer);
    procedure StyleChanged(Sender: TObject);
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMIsToolControl(var Message: TMessage); message CM_ISTOOLCONTROL;
    procedure WMWindowPosChanged(var Message: TWMWindowPosChanged); message WM_WINDOWPOSCHANGED;
    procedure SetAlignment(Value: TAlignment);
    procedure CMDockClient(var Message: TCMDockClient); message CM_DOCKCLIENT;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    procedure Paint; override;
    property FullRepaint: Boolean read FFullRepaint write FFullRepaint default True;
    property Locked: Boolean read FLocked write FLocked default False;
    property ParentColor default False;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetControlsAlignment: TAlignment; override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property Brush: TBrush read FBrush write SetBrush;
    property Pen: TPen read FPen write SetPen;
    property Arc: Integer read FArc write SetArc default 100;
    property BackgroundColor: TColor read FColor write FColor;
  end;


  TRoundPanel = class(TCustomRoundPanel)
  public
    property DockManager;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('AWRDS', [TRoundPanel]);
end;


{ TCustomRoundPanel }

constructor TCustomRoundPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
                   csOpaque, csDoubleClicks, csReplicatable];
  Width := 300;
  Height := 300;
  FPen := TPen.Create;
  FPen.OnChange := StyleChanged;
  FBrush := TBrush.Create;
  FBrush.OnChange := StyleChanged;
  FBrush.Color := 15329769; // a somewhat light-grey
  FColor := $00C08000;      // "off-blue"
  SetArc(100);
  Anchors := [akLeft, akTop, akRight, akBottom];

  FAlignment := taCenter;
  FFullRepaint := True;
  UseDockManager := True;
end;

destructor TCustomRoundPanel.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TCustomRoundPanel.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
     if NewStyleControls and Ctl3D then
        Style := Style and not WS_BORDER;

     WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
  end;
end;

procedure TCustomRoundPanel.CMCtl3DChanged(var Message: TMessage);
begin
  RecreateWnd;
  inherited;
end;

procedure TCustomRoundPanel.CMIsToolControl(var Message: TMessage);
begin
  if not FLocked then Message.Result := 1;
end;

procedure TCustomRoundPanel.WMWindowPosChanged(var Message: TWMWindowPosChanged);
begin
  if FullRepaint then
     Invalidate;
  inherited;
end;

procedure TCustomRoundPanel.SetAlignment(Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

function TCustomRoundPanel.GetControlsAlignment: TAlignment;
begin
  Result := FAlignment;
end;

procedure TCustomRoundPanel.CMDockClient(var Message: TCMDockClient);
var
  R: TRect;
  Dim: Integer;
begin
  if AutoSize then
  begin
    FAutoSizeDocking := True;
    try
      R := Message.DockSource.Control.BoundsRect;
      case Align of
        alLeft: if Width = 0 then Width := R.Right - R.Left;
        alRight: if Width = 0 then
          begin
            Dim := R.Right - R.Left;
            SetBounds(Left - Dim, Top, Dim, Height);
          end;
        alTop: if Height = 0 then Height := R.Bottom - R.Top;
        alBottom: if Height = 0 then
          begin
            Dim := R.Bottom - R.Top;
            SetBounds(Left, Top - Dim, Width, Dim);
          end;
      end;
      inherited;
      Exit;
    finally
      FAutoSizeDocking := False;
    end;
  end;
  inherited;
end;


function TCustomRoundPanel.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
begin
  Result := (not FAutoSizeDocking) and inherited CanAutoSize(NewWidth, NewHeight);
end;


procedure TCustomRoundPanel.Paint;
var
  Rect: TRect;
  X, Y, W, H: Integer;
begin
  Rect := GetClientRect;
  with Canvas do
  begin
    Brush.Color := FColor;
    FillRect(Rect);
    Pen := FPen;
    Brush := FBrush;
    X := Pen.Width div 2;
    Y := X;
    W := Width - Pen.Width + 1;
    H := Height - Pen.Width + 1;
    if Pen.Width = 0 then
    begin
       Dec(W);
       Dec(H);
    end;
    RoundRect(X, Y, X + W, Y + H, FArc, FArc);
  end;
end;

procedure TCustomRoundPanel.StyleChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TCustomRoundPanel.SetBrush(Value: TBrush);
begin
  FBrush.Assign(Value);
end;

procedure TCustomRoundPanel.SetPen(Value: TPen);
begin
  FPen.Assign(Value);
end;

procedure TCustomRoundPanel.SetArc(Value: Integer);
begin
  if FArc <> Value then
  begin
    FArc := Value;
    Invalidate;
  end;
end;

end.
